// 
// The Waveset SPML General License 
// 
// Version 0.1, April 2003
// Copyright (C) 2003 Waveset Technologies, Inc..
// 6034 West Courtyard Drive, Suite 210, Austin, Texas 78730
// All rights reserved.
// 
// TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions, and the disclaimers in Sections 6
//    and 7 below.
// 
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the disclaimers in Sections 6
//    and 7 below in the documentation and/or other materials provided
//    with the distribution.
// 
// 3. The end-user documentation included with the redistribution, if
//    any, must include the following acknowledgment:
// 
//    "This product includes software developed by 
//     Waveset Technologies, Inc. (www.waveset.com)."
// 
//    Alternately, this acknowledgment may appear in the software itself, if
//    and wherever such third-party acknowledgments normally appear.
//  
// 4. The names "Waveset" and "Waveset Technologies, Inc." must not be
//    used to endorse or promote products derived from this software
//    without the prior written permission of Waveset. For written
//    permission, please contact www.waveset.com.
//  
// 5. Products derived from this software may not be called "Waveset",
//    nor may "Waveset" appear in their name, without the prior written
//    permission of Waveset.
//  
// 6. NO WARRANTY 
// 
//    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED
//    OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//    DISCLAIMED.
// 
// 7.  LIMITATION OF LIABILITY
// 
//    IN NO EVENT SHALL THE WAVESET OR ITS LICENSORS BE LIABLE FOR ANY
//    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
//    GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
//    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
//    IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
//    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
// End of Terms and Conditions
// ====================================================================
//
// Component Name: org.openspml.message.ExtendedRequest
//
// Author(s): Jeff Larson
//
// Description:
//
// Memory representation for an <extendedRequest> element.
//
//

package org.openspml.message;

import java.util.*;
import org.openspml.util.*;

/**
 * A class used to represent an <code>extendedRequest</code> element.
 */
public class ExtendedRequest extends SpmlRequest {

    //////////////////////////////////////////////////////////////////////
    //
    // Fields
    //
    // Also inherit an <idetnifier> from SpmlRequest which presumably
    // is the identifier of the target object.
    //
    //////////////////////////////////////////////////////////////////////

    static final String ELEMENT = "extendedRequest";

    /**
     * Provider identifier.     
     */
    ProviderIdentifier _providerId;

    /**
     * Operation identifier.
     */
    OperationIdentifier _operationId;

    /**
     * A list of Attribute objects.
     */
    List _attributes;

    //////////////////////////////////////////////////////////////////////
    //
    // Constructors
    //
    //////////////////////////////////////////////////////////////////////

    public ExtendedRequest() {
    }

    ExtendedRequest(XmlElement e) {
        parseXml(e);
    }

    public SpmlResponse createResponse() {
        return new ExtendedResponse();
    }

    public void setProviderIdentifier(ProviderIdentifier id) {
        _providerId = id;
    }

    public void setProviderIdentifier(String id) {
        _providerId = new ProviderIdentifier(id);
    }

    public void setOperationIdentifier(OperationIdentifier id) {
        _operationId = id;
    }

    public void setOperationIdentifier(String id) {
        _operationId = new OperationIdentifier(id);
    }

    public void setAttributes(List l) {
        _attributes = l;
    }

    public void setAttribute(Attribute a) {
        if (_attributes == null)
            _attributes = new ArrayList();
        _attributes.add(a);
    }
    
    public void setAttribute(String name, Object value) {
        setAttribute(new Attribute(name, value));
    }

    //////////////////////////////////////////////////////////////////////
    //
    // Accessors
    //
    //////////////////////////////////////////////////////////////////////

    public ProviderIdentifier getProviderIdentifier() {
        return _providerId;
    }

    public OperationIdentifier getOperationIdentifier() {
        return _operationId;
    }

    public List getAttributes() {
        return _attributes;
    }

    public Attribute getAttribute(String name) {
	return Attribute.getAttribute(_attributes, name);
    }

    public Object getAttributeValue(String name) {
	return Attribute.getAttributeValue(_attributes, name);
    }

    //////////////////////////////////////////////////////////////////////
    //
    // XML
    //
    //////////////////////////////////////////////////////////////////////

    public String getElementName() {
        return ELEMENT;
    }

    void addSubclassElements(SpmlBuffer b) {

        if (_providerId != null)
            _providerId.toXml(b);

        if (_operationId != null)
            _operationId.toXml(b);

        if (_attributes != null)
            Attribute.toXml(b, "attributes", _attributes);
    }

    void parseXml(XmlElement e) {
        super.parseXml(e);

	for (XmlElement child = e.getChildElement() ; 
	     child != null ; child = child.next()) {
            String tag = child.getLocalName();

	    if (tag.equals(ProviderIdentifier.ELEMENT))
		_providerId = new ProviderIdentifier(child);

	    else if (tag.equals(OperationIdentifier.ELEMENT))
		_operationId = new OperationIdentifier(child);

            else if (tag.equals("attributes"))
                _attributes = Attribute.parseList(child);
	}
    }

}
